import logging
import dbus
from .Core.errors import *
from .Core.enums import *
from SystemUpdater.constants import PUTSH_CONTENT_LOCATION
from gettext import gettext as _

class UpdateContent:
    def __init__(self, _repo_name=None, _branch=None, _revision=None):
        self.repo_name = _repo_name
        self.branch = _branch
        self.revision = _revision

#更新important.list的本次升级的列表
def update_important(bus,reply_handler,error_handler):
    def _update_important_reply(retval):
        if bool(retval) == False:
            error_handler(ERROR_UPDATE_SOURCE_FAILED)
        else:
            reply_handler()
    def _update_important_error(retval):
        logging.error(str(retval))
        error_handler(ERROR_UPDATE_SOURCE_FAILED)
    
    try:
        obj = bus.get_object('com.kylin.software.properties', '/com/kylin/software/properties')

        interface = dbus.Interface(obj, dbus_interface='com.kylin.software.properties.interface')
        interface.updateSourceTemplate(timeout=20,reply_handler=_update_important_reply,error_handler=_update_important_error)
    except Exception as e:
        logging.warning(str(e))
        raise UpdateBaseError(ERROR_UPDATE_SOURCE_FAILED)

def _read_important_data():
    # 当存在源管理推送内容时 优先从源管理获取当前推送的分支信息 
    try:
        push_list = []
        with open(PUTSH_CONTENT_LOCATION, 'r') as f:
            data = f.read()
            push_list = data.split()

        #去除重复内容
        push_list = list(set(push_list))

        logging.info("Server push list: %r",push_list)
        return push_list
    except Exception as e:
        logging.warning(str(e))
        return []

def _parse_push_content(push_content):
    # 推送的内容格式 例如： kylin:kylin/2203/230@commit 号
    update_content = UpdateContent()
    try:
        refspec = push_content.split(":")
        if len(refspec) == 2:
            update_content.repo_name = refspec[0]
            ref_and_rev = refspec[1]
            if "@" in ref_and_rev:
                refs_details = ref_and_rev.split("@")
                update_content.branch = refs_details[0]
                update_content.revision = refs_details[1]
            else:
                update_content.branch = ref_and_rev
        return update_content
    except Exception as e:
        logging.warning(str(e))
        return push_content

def get_push_content():
    update_list = []
    push_list = _read_important_data()
    for cont in push_list:
        update_cont = _parse_push_content(cont)
        update_list.append(update_cont)
    return update_list
